/*----- PROTECTED REGION ID(TopupSB.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        TopupSB.cpp
//
// description : C++ source for the TopupSB and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               TopupSB are implemented in this file.
//
// project :     TopupSB.
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================

#include <tango.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "fb_thread.h"
#include <TopupSB.h>
#include <TopupSBClass.h>

/*----- PROTECTED REGION END -----*/


/**
 *	TopupSB class description:
 *	Topup in single bunch
 */

//================================================================
//
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name  |  Method name
//----------------------------------------------------------------
//  State         |  Inherited (no method)
//  Status        |  Inherited (no method)
//  On            |  on
//  Off           |  off
//  Reset         |  reset
//  Standby       |  standby
//================================================================

namespace TopupSB_ns
{
	/*----- PROTECTED REGION ID(TopupSB::namespace_starting) ENABLED START -----*/

	//	static initializations

	/*----- PROTECTED REGION END -----*/	//	TopupSB::namespace_starting



//--------------------------------------------------------
/**
 *	Method      : TopupSB::TopupSB()
 *	Description : Constructors for a Tango device
 *	              implementing the class TopupSB
 */
//--------------------------------------------------------
TopupSB::TopupSB(Tango::DeviceClass *cl, string &s)
 	: Tango::Device_4Impl(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(TopupSB::constructor_1) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	TopupSB::constructor_1
}
//--------------------------------------------------------
TopupSB::TopupSB(Tango::DeviceClass *cl, const char *s)
 	: Tango::Device_4Impl(cl, s)
{
	/*----- PROTECTED REGION ID(TopupSB::constructor_2) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	TopupSB::constructor_2
}
//--------------------------------------------------------
TopupSB::TopupSB(Tango::DeviceClass *cl, const char *s, const char *d)
 	: Tango::Device_4Impl(cl, s, d)
{
	/*----- PROTECTED REGION ID(TopupSB::constructor_3) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	TopupSB::constructor_3
}


//--------------------------------------------------------
/**
 *	Method      : TopupSB::delete_device()()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void TopupSB::delete_device()
{
	/*----- PROTECTED REGION ID(TopupSB::delete_device) ENABLED START -----*/

	//	Delete device allocated objects

	/*----- PROTECTED REGION END -----*/	//	TopupSB::delete_device
	delete[] attr_SbPosTMBF_read;
	delete[] attr_SbPtpTMBF_read;
	delete[] attr_SbPosFillpath_read;
	delete[] attr_NumberTurn_read;
	delete[] attr_NumberSb_read;
	delete[] attr_Delta_read;
	delete[] attr_Plane_read;
	delete[] attr_ProgrammeBucket_read;
	delete[] attr_Progress_read;
    delete[] attr_PatternScope_read;
	
	delete DcctD;	
	delete FillpathD;
	delete GuntimingD;
	delete TopupD; 
     delete ScopeD;    
	
	abortflag = true;

	topupsb_stat[TOPUPSB_THREAD_ON] = false;
	topupsb_stat[TOPUPSB_THREAD_STANDBY] = false;	

}


//--------------------------------------------------------
/**
 *	Method      : TopupSB::init_device()
 *	Description : //	will be called at device initialization.
 */
//--------------------------------------------------------
void TopupSB::init_device()
{
	DEBUG_STREAM << "TopupSB::init_device() create device " << device_name << endl;

	/*----- PROTECTED REGION ID(TopupSB::init_device_before) ENABLED START -----*/

	//	Initialization before get_device_property() call

	/*----- PROTECTED REGION END -----*/	//	TopupSB::init_device_before
	
	//	Get the device properties (if any) from database
      get_device_property();
      
      attr_SbPosTMBF_read = new Tango::DevDouble[1];
      attr_SbPtpTMBF_read = new Tango::DevDouble[1];
      attr_SbPosFillpath_read = new Tango::DevDouble[1];
      attr_NumberTurn_read = new Tango::DevDouble[1];
      attr_NumberSb_read = new Tango::DevDouble[1];
      attr_Delta_read = new Tango::DevDouble[1];
      attr_Plane_read = new Tango::DevDouble[1];
      attr_ProgrammeBucket_read = new Tango::DevLong[432];
      attr_Progress_read = new Tango::DevBoolean[5];
      attr_PatternScope_read = new Tango::DevDouble[40000];
      
      double fillpathSB = 0;
      
      for (int i = 0; i < 432; i++) { 
            pattern[i] = 0;      
      }
      
      for (int i = 0; i < 5; i++) { 
            progresso[i] = 0;      
      }

      abortflag = false;

      DcctD = new Tango::DeviceProxy(dcctDevice);
      FillpathD = new Tango::DeviceProxy(fillpathDevice);	
      GuntimingD = new Tango::DeviceProxy(guntimingDevice);
      TopupD = new Tango::DeviceProxy(topupDevice);
      ScopeD = new Tango::DeviceProxy(scopeDevice);
      try     {
                  ScopeD->set_timeout_millis(60000);
            }
      catch(Tango::DevFailed e) {
            Tango::Except::throw_exception(
            (const char *)"Failed to set timeout ScopeD",
            (const char *)"Error from profile server",
            (const char *)"TopupSB::ScopeD()", Tango::ERR);   
            }  
      
      
      fb_loop = new fb_thread(this);
      fb_loop->start();

	
	/*----- PROTECTED REGION ID(TopupSB::init_device) ENABLED START -----*/

	//	Initialize device

	/*----- PROTECTED REGION END -----*/	//	TopupSB::init_device
}



//--------------------------------------------------------
/**
 *	Method      : TopupSB::get_device_property()
 *	Description : //	Add your own code to initialize
 */
//--------------------------------------------------------
void TopupSB::get_device_property()
{
	/*----- PROTECTED REGION ID(TopupSB::get_device_property_before) ENABLED START -----*/

	//	Initialize property data members

	/*----- PROTECTED REGION END -----*/	//	TopupSB::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("DcctDevice"));
	dev_prop.push_back(Tango::DbDatum("FillpathDevice"));
	dev_prop.push_back(Tango::DbDatum("LoopPeriod"));
	dev_prop.push_back(Tango::DbDatum("Min"));
	dev_prop.push_back(Tango::DbDatum("Max"));
	dev_prop.push_back(Tango::DbDatum("PlaneGet"));
	dev_prop.push_back(Tango::DbDatum("GuntimingDevice"));
	dev_prop.push_back(Tango::DbDatum("TopupDevice"));
     dev_prop.push_back(Tango::DbDatum("ScopeDevice"));
	dev_prop.push_back(Tango::DbDatum("NumberTurn"));
	dev_prop.push_back(Tango::DbDatum("NumberSb"));
	dev_prop.push_back(Tango::DbDatum("Delta"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on TopupSBClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		TopupSBClass	*ds_class =
			(static_cast<TopupSBClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize DcctDevice from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dcctDevice;
		else {
			//	Try to initialize DcctDevice from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dcctDevice;
		}
		//	And try to extract DcctDevice value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dcctDevice;

		//	Try to initialize FillpathDevice from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  fillpathDevice;
		else {
			//	Try to initialize FillpathDevice from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  fillpathDevice;
		}
		//	And try to extract FillpathDevice value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  fillpathDevice;

		//	Try to initialize LoopPeriod from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  loopPeriod;
		else {
			//	Try to initialize LoopPeriod from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  loopPeriod;
		}
		//	And try to extract LoopPeriod value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  loopPeriod;

		//	Try to initialize Min from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  min;
		else {
			//	Try to initialize Min from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  min;
		}
		//	And try to extract Min value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  min;

		//	Try to initialize Max from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  max;
		else {
			//	Try to initialize Max from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  max;
		}
		//	And try to extract Max value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  max;
		
		//	Try to initialize PlaneGet from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  planeGet;
		else {
			//	Try to initialize PlaneGet from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  planeGet;
		}
		//	And try to extract PlaneGet value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  planeGet;

		//	Try to initialize GuntimingDevice from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  guntimingDevice;
		else {
			//	Try to initialize GuntimingDevice from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  guntimingDevice;
		}
		//	And try to extract GuntimingDevice value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  guntimingDevice;

		//	Try to initialize TopupDevice from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  topupDevice;
		else {
			//	Try to initialize TopupDevice from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  topupDevice;
		}
		//	And try to extract TopupDevice value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  topupDevice;

		// Try to initialize ScopeDevice from class property
          cl_prop = ds_class->get_class_property(dev_prop[++i].name);
          if (cl_prop.is_empty()==false)     cl_prop  >>  scopeDevice;
          else {
               //   Try to initialize ScopeDevice from default device value
               def_prop = ds_class->get_default_device_property(dev_prop[i].name);
               if (def_prop.is_empty()==false)    def_prop  >>  scopeDevice;
          }
          //   And try to extract ScopeDevice value from database
          if (dev_prop[i].is_empty()==false) dev_prop[i]  >>  scopeDevice;
          
          //	Try to initialize NumberTurn from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  numberTurn;
		else {
			//	Try to initialize NumberTurn from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  numberTurn;
		}
		//	And try to extract NumberTurn value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  numberTurn;

          //	Try to initialize NumberSb from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  numberSb;
		else {
			//	Try to initialize NumberSb from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  numberSb;
		}
		//	And try to extract NumberSb value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  numberSb;

          //	Try to initialize Delta from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  delta;
		else {
			//	Try to initialize Delta from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  delta;
		}
		//	And try to extract Delta value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  delta;


	}
	/*----- PROTECTED REGION ID(TopupSB::get_device_property_after) ENABLED START -----*/

	//	Check device property data members init

	/*----- PROTECTED REGION END -----*/	//	TopupSB::get_device_property_after

}

//--------------------------------------------------------
/**
 *	Method      : TopupSB::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void TopupSB::always_executed_hook()
{
	set_devstat();
}



//--------------------------------------------------------
/**
 *	Method      : TopupSB::read_attr_hardware()
 *	Description : Hardware acquisition for attributes.
 */
//--------------------------------------------------------
void TopupSB::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "TopupSB::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_attr_hardware) ENABLED START -----*/

	//	Add your own code

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_attr_hardware

}


//--------------------------------------------------------
/**
 *	Read SbPosTMBF attribute
 *	Description: single bunch position from TMBF
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_SbPosTMBF(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_SbPosTMBF(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_SbPosTMBF) ENABLED START -----*/

	//	Set the attribute value
	*attr_SbPosTMBF_read = posTMBF;
	attr.set_value(attr_SbPosTMBF_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_SbPosTMBF
}

//--------------------------------------------------------
/**
 *	Write SbPosTMBF attribute values to hardware.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::write_SbPosTMBF(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_SbPosTMBF(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_SbPosTMBF) ENABLED START -----*/
	posTMBF = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_SbPosTMBF
}

//--------------------------------------------------------
/**
 *	Read SbPtpTMBF attribute
 *	Description: single bunch ptp from TMBF
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_SbPtpTMBF(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_SbPtpTMBF(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_SbPtpTMBF) ENABLED START -----*/

	//	Set the attribute value
	*attr_SbPtpTMBF_read = ptpTMBF;
	attr.set_value(attr_SbPtpTMBF_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_SbPtpTMBF
}
//--------------------------------------------------------
/**
 *	Read SbPosFillpath attribute
 *	Description: single bunch position from Fillpath
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_SbPosFillpath(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_SbPosFillpath(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_SbPosFillpath) ENABLED START -----*/

	*attr_SbPosFillpath_read = fillpathSB;
	attr.set_value(attr_SbPosFillpath_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_SbPosFillpath
}

void TopupSB::write_SbPosFillpath(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_SbPosFillpath(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_SbPosTMBF) ENABLED START -----*/
	fillpathSB = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_SbPosTMBF
}

//--------------------------------------------------------
/**
 *	Read NumberTurn attribute
 *	Description: number of turn to aquire
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_NumberTurn(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_NumberTurn(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_NumberTurn) ENABLED START -----*/

	//	Set the attribute value
	*attr_NumberTurn_read = numberTurn;
	attr.set_value(attr_NumberTurn_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_NumberTurn
}

//--------------------------------------------------------
/**
 *	Write NumberTurn attribute values to hardware.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::write_NumberTurn(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_NumberTurn(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_NumberTurn) ENABLED START -----*/
	numberTurn = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_NumberTurn
}

//--------------------------------------------------------
/**
 *	Read Plane attribute
 *	Description: H or V TMBF
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_Plane(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_Plane(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_Plane) ENABLED START -----*/

	//	Set the attribute value
	*attr_Plane_read = planeGet;
	attr.set_value(attr_Plane_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_NumberTurn
}

//--------------------------------------------------------
/**
 *	Write Plane attribute values to hardware.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::write_Plane(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_Plane(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_NumberTurn) ENABLED START -----*/
	planeGet = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_NumberTurn
}

//--------------------------------------------------------
/**
 *	Read NumberSb attribute
 *	Description: How many SB
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_NumberSb(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_NumberSb(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_NumberSb) ENABLED START -----*/

	//	Set the attribute value
	*attr_NumberSb_read = numberSb;
	attr.set_value(attr_NumberSb_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_NumberTurn
}

//--------------------------------------------------------
/**
 *	Write NumberSb attribute values to hardware.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::write_NumberSb(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_NumberTurn(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_NumberTurn) ENABLED START -----*/
	numberSb = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_NumberTurn
}

//--------------------------------------------------------
/**
 *	Read Delta attribute
 *	Description: How many SB
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::read_Delta(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_NumberSb(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_Delta) ENABLED START -----*/

	//	Set the attribute value
	*attr_Delta_read = delta;
	attr.set_value(attr_Delta_read);

	/*----- PROTECTED REGION END -----*/	//	TopupSB::read_Delta
}

//--------------------------------------------------------
/**
 *	Write Delta attribute values to hardware.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar 
 */
//--------------------------------------------------------
void TopupSB::write_Delta(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TopupSB::write_Delta(Tango::Attribute &attr) entering... " << endl;
	
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	
	/*----- PROTECTED REGION ID(TopupSB::write_Delta) ENABLED START -----*/
	delta = w_val;
	

	/*----- PROTECTED REGION END -----*/	//	TopupSB::write_NumberTurn
}

//--------------------------------------------------------
/**
 *	Read ProgrammeBucket attribute
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Spectrum  max = 432
 */
//--------------------------------------------------------
void TopupSB::read_ProgrammeBucket(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_ProgrammeBucket(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(TopupSB::read_ProgrammeBucket) ENABLED START -----*/
	
	attr.set_value(pattern, 432);

}

//--------------------------------------------------------
/**
 *  Read PatternScope attribute
 *  Description: 
 *
 *  Data type:  Tango::DevLong
 *  Attr type:  Spectrum  max = 432
 */
//--------------------------------------------------------
void TopupSB::read_PatternScope(Tango::Attribute &attr)
{
    DEBUG_STREAM << "TopupSB::read_PatternScope(Tango::Attribute &attr) entering... " << endl;
   
    attr.set_value(patternScope, 40000);

}

//--------------------------------------------------------
/**
 *	Read Progress attribute
 *	Description: 
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Spectrum  max = 432
 */
//--------------------------------------------------------
void TopupSB::read_Progress(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TopupSB::read_Progress(Tango::Attribute &attr) entering... " << endl;
	
	attr.set_value(progresso, 5);

}

//--------------------------------------------------------
/**
 *	Method      : TopupSB::TopupSBClass::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *	              for specified device.
 */
//--------------------------------------------------------
void TopupSB::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(TopupSB::Class::add_dynamic_attributes) ENABLED START -----*/

	//	Add your own code to create and add dynamic attributes if any

	/*----- PROTECTED REGION END -----*/	//	TopupSB::Class::add_dynamic_attributes

}



//========================================================
//	Command execution methods
//========================================================

//--------------------------------------------------------
/**
 *	Execute the On command:
 *	Description: 
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
void TopupSB::on()
{
	DEBUG_STREAM << "TopupSB::On()  - " << device_name << endl;

	abortflag = false;
	std::stringstream s;	
	topupsb_stat[TOPUPSB_DCCT_CURR] = false;
	topupsb_stat[TOPUPSB_THREAD_CORRECTION] = false;
	topupsb_stat[TOPUPSB_THREAD_ERR_THRESHOLD] = false;

	if (topupsb_stat[TOPUPSB_THREAD_STANDBY] == false) {
		abortflag = false;		
		fb_loop = new fb_thread(this);
		if(fb_loop == 0) {
			Tango::Except::throw_exception (
				(const char *)"OUT OF MEMORY",
				(const char *)"out of memory error",
				(const char *)"TopupSB::on()",Tango::ERR);
		}		
		
		topupsb_stat[TOPUPSB_THREAD_ON] = true;
		fb_loop->start();
	}
	else {
		abortflag = true; /* exit from long sleeping */
		topupsb_stat[TOPUPSB_THREAD_ON] = true;
		topupsb_stat[TOPUPSB_THREAD_STANDBY] = true;
	}
	topupsb_stat[TOPUPSB_THREAD_ON] = true;
	topupsb_stat[TOPUPSB_THREAD_STANDBY] = true;

}

//--------------------------------------------------------
/**
 *	Execute the Off command:
 *	Description: 
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
void TopupSB::off()
{
	DEBUG_STREAM << "TopupSB::Off()  - " << device_name << endl;

	std::stringstream s;
	abortflag = true;
	topupsb_stat[TOPUPSB_THREAD_ON] = false;
	topupsb_stat[TOPUPSB_THREAD_STANDBY] = false;	
	topupsb_stat[TOPUPSB_DCCT_CURR] = false;
	topupsb_stat[TOPUPSB_THREAD_CORRECTION] = false;
	topupsb_stat[TOPUPSB_THREAD_ERR_THRESHOLD] = false;

}

//--------------------------------------------------------
/**
 *	Execute the Reset command:
 *	Description: 
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
void TopupSB::reset()
{
	DEBUG_STREAM << "TopupSB::Reset()  - " << device_name << endl;

	std::stringstream s;
	
	topupsb_stat[TOPUPSB_THREAD_ON] == false;
	topupsb_stat[TOPUPSB_THREAD_STANDBY] == false;
	topupsb_stat[TOPUPSB_DCCT_CURR] = false;
	topupsb_stat[TOPUPSB_THREAD_CORRECTION] = false;
	topupsb_stat[TOPUPSB_THREAD_ERR_THRESHOLD] = false;
	topupsb_stat[TOPUPSB_THREAD_ERR] = false;

}

//--------------------------------------------------------
/**
 *	Execute the Standby command:
 *	Description: 
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
void TopupSB::standby()
{
	DEBUG_STREAM << "TopupSB::Standby()  - " << device_name << endl;

	std::stringstream s;
	abortflag = false;

	if (topupsb_stat[TOPUPSB_THREAD_STANDBY] == false) {
		abortflag = false;
		fb_loop = new fb_thread(this);
		if(fb_loop == 0) {
			Tango::Except::throw_exception (
				(const char *)"OUT OF MEMORY",
				(const char *)"out of memory error",
				(const char *)"TopupSB::standby()",Tango::ERR);
		}			
		fb_loop->start();
	}	
		
	topupsb_stat[TOPUPSB_THREAD_STANDBY] = true;
	topupsb_stat[TOPUPSB_THREAD_ON] = false;

}

//+------------------------------------------------------------------
/**
 *	method:	TOPUPSB::set_devstat
 *
 *	Set device status 
 */
//+------------------------------------------------------------------
void TopupSB::set_devstat(void) 
{
	std::stringstream s;


	if ((topupsb_stat[TOPUPSB_THREAD_ON] == true) && 
	    	(topupsb_stat[TOPUPSB_THREAD_STANDBY] == true))  {
		set_state(Tango::ON);
		s << "ON: feedback abilitato" << endl;
	}

	if ((topupsb_stat[TOPUPSB_THREAD_ON] == false) && 
	    	(topupsb_stat[TOPUPSB_THREAD_STANDBY] == true))  {
		set_state(Tango::STANDBY);
		s << "STANDBY:  feedback in standby. Acquisizione ogni 10min" << endl;
	}

	if ((topupsb_stat[TOPUPSB_THREAD_ON] == false) && 
	    	(topupsb_stat[TOPUPSB_THREAD_STANDBY] == false) &&
		(topupsb_stat[TOPUPSB_THREAD_ERR] == false))  {
		set_state(Tango::OFF);
		s << "OFF:  feedback disabilitato" << endl;
	}	
	
	if (topupsb_stat[TOPUPSB_DCCT_ERR]) {
		s << "SR DCCT errore di lettura" << endl;
          set_state(Tango::FAULT);
	}
	
	if (topupsb_stat[TOPUPSB_DCCT_CURR]) {
		s << "Corrente inferiore ai 20mA per agganciare il loop" << endl;
          set_state(Tango::FAULT);
	}

	if (topupsb_stat[TOPUPSB_FILLPATH_ERR]) {
		s << "Errore durante il setup del Fillpat" << endl;
          set_state(Tango::FAULT);
	}
    if (topupsb_stat[TOPUPSB_FILLPAT_COMPARE]) {
        s << "SB settato nel TopupSB e Fillpat non corrispondenti" << endl;
          set_state(Tango::FAULT);
    }
    
         
     if (topupsb_stat[TOPUPSB_SB_SET]) {
           s << "Single bunch non impostato" << endl;
          set_state(Tango::FAULT);
     }
     if (topupsb_stat[TOPUPSB_WAIT]) {
            s << "In attesa dell'iniezione in MB ..." << endl;
            set_state(Tango::MOVING);
         }
     if (topupsb_stat[TOPUPSB_WAIT_MAX]) {
            s << "Prossima iniezione distante, si attende prima di impostare i parametri SB ..." << endl;
          set_state(Tango::ON);
         }        	

	if (topupsb_stat[TOPUPSB_THREAD_CORRECTION]) {
		s << "Impostando i parametri SB ..." << endl;
          set_state(Tango::MOVING);        
	}
     if (topupsb_stat[TOPUPSB_INJ]) {
          s << "Iniezione in corso ..." << endl;
          set_state(Tango::MOVING);        
     }
     
     if (topupsb_stat[TOPUPSB_WAIT_INJ]) {
          s << "In attesa che l'iniezione inizi ..." << endl;
          set_state(Tango::MOVING);
     }
     
     if (topupsb_stat[TOPUPSB_ERR_SCOPE]) {
          s << "Errore leggendo l'oscilloscopio del riempimento ..." << endl;
          set_state(Tango::FAULT);
     }    
     
     
     if (topupsb_stat[TOPUPSB_NORMAL]) {
          s << "Impostando i parametri MB ..." << endl;
          set_state(Tango::MOVING);
     }
	
	if (topupsb_stat[TOPUPSB_GRID_ERR]) {
		s << "Errore durante il setup della griglia del GUN ..." << endl;
          set_state(Tango::FAULT);
	}
    if (topupsb_stat[TOPUPSB_MAXCURRENT_ERR]) {
        s << "Errore durante il setup della corrente finale dell'iniezione in Topup ..." << endl;
          set_state(Tango::FAULT);
    }
		if (topupsb_stat[TOPUPSB_TOPUP_ERR]) {
		s << "Errore in lettura dello stato del Topup ..." << endl;
          set_state(Tango::FAULT);
	}
    if (topupsb_stat[TOPUPSB_CURR_HIGH]) {
        s << "Raggiunto limite massima di corrente in SB ..." << endl;
          set_state(Tango::ALARM);
    }
	
	if (topupsb_stat[TOPUPSB_GUNTIMING_ERR]) {
		s << "Errore cambiando la modalita' del GUN ..." << endl;
          set_state(Tango::FAULT);        
	}

	if (topupsb_stat[TOPUPSB_TOO_LONG]) {
		s << "Iniezione in SB troppo lenta. Verificare l'accumulo in SB ..." << endl;
          set_state(Tango::FAULT);        
	}
	
	if (topupsb_stat[TOPUPSB_THREAD_SLEEP]) {
		s << "In attesa del prossimo controllo ..." << endl;
           set_state(Tango::ON);        
	}	
		
	if (	topupsb_stat[TOPUPSB_DCCT_CURR] ||
		topupsb_stat[TOPUPSB_THREAD_ERR] ||
		topupsb_stat[TOPUPSB_DCCT_CURR] || 
		topupsb_stat[TOPUPSB_THREAD_ERR_THRESHOLD]) {
		set_state(Tango::FAULT);
	}
	set_status(s.str());	

}	

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::Multibunch
 *
 *	description:	method to execute "Multibunch"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::MultibunchOff()
{
	Tango::DeviceAttribute Din("MultiBunch",false);
	
	try {	
		GuntimingD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Writing new Gun MB or SB state " <<  GuntimingD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::GuntimingD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

void TopupSB::MultibunchOn()
{
	Tango::DeviceAttribute Din("MultiBunch",true);
	
	try {	
		GuntimingD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Writing new Gun MB or SB state " <<  GuntimingD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::GuntimingD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::Automode
 *
 *	description:	method to execute "Automode"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::AutomodeOn()
{
	Tango::DeviceAttribute Din("Automode",true);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable Automode " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}
void TopupSB::AutomodeOff()
{
	Tango::DeviceAttribute Din("Automode",false);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable Automode " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}



//+------------------------------------------------------------------
/**
 *	method:	TopupSB::Loopmode
 *
 *	description:	method to execute "Loopmode"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::LoopmodeOn()
{
	Tango::DeviceAttribute Din("Loopmode",true);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable Loopmode " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}
void TopupSB::LoopmodeOff()
{
	Tango::DeviceAttribute Din("Loopmode",false);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable Loopmode " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::Keepbooster
 *
 *	description:	method to execute "Keepbooster"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::KeepboosterOn()
{
	Tango::DeviceAttribute Din("KeepBoosterCurrent",true);
	try {	
		TopupD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable KeepboosterCurrent " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}
void TopupSB::KeepboosterOff()
{
	Tango::DeviceAttribute Din("KeepBoosterCurrent",false);
	try {	
		TopupD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable KeepboosterCurrent " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::Enable SB
 *
 *	description:	method to execute "Enable SB"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::EnablesbOn()
{
	Tango::DeviceAttribute Din("EnableTopupSB",true);
	try {	
		TopupD->command_inout("Off");
		abort_sleep(1);		
		TopupD->write_attribute(Din);
		TopupD->command_inout("On");
		abort_sleep(1);		
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable SB " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}
void TopupSB::EnablesbOff()
{
	Tango::DeviceAttribute Din("EnableTopupSB",false);
	try {	
		TopupD->command_inout("Off");
		abort_sleep(1);		
		TopupD->write_attribute(Din);
		TopupD->command_inout("On");
		abort_sleep(1);		
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Enable or disable SB " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::MaxGrid
 *
 *	description:	method to execute "MaxGrid"
 *	
 *
 *
 */
//+------------------------------------------------------------------
int TopupSB::MaxGrid(double mode)
{
	Tango::DeviceAttribute Din("MaxGridVoltage",mode);
	try {	
		TopupD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Set MaxGrid " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::MinGrid
 *
 *	description:	method to execute "MinGrid"
 *	
 *
 *
 */
//+------------------------------------------------------------------
int TopupSB::MinGrid(double mode)
{
	Tango::DeviceAttribute Din("MinGridVoltage",mode);
	try {	
		TopupD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Set MinGrid " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *  method: TopupSB::MaxCurrent
 *
 *  description:    method to execute "MaxGrid"
 *  
 *
 *
 */
//+------------------------------------------------------------------
int TopupSB::MaxCurrent(double mode)
{
    Tango::DeviceAttribute Din("CurrentMax",mode);
    try {   
        TopupD->write_attribute(Din);
    }
    catch(Tango::DevFailed e) {
        TangoSys_MemStream out_stream;
        out_stream << "Set MaxCurrent " <<  TopupD << " failed, err=" << e.errors[0].desc << ends;
        WARN_STREAM << "TopupSB::TopupD(): " <<  out_stream.str() << endl;
        set_state(Tango::FAULT);    
        set_status(out_stream.str());
        abort_sleep(1);     
    }
}

int TopupSB::topupState()
{
	Tango::DevState data;
	Tango::DeviceAttribute Dout;
	try {
		Dout = TopupD->read_attribute("State");
		Dout >> data;
		if (data == Tango::RUNNING)
			return 1;
		else
			return 0;
	}
	catch(Tango::DevFailed e) {
		Tango::Except::throw_exception(
			(const char *)"Failed to read mps status",
			(const char *)"Error from mps server",
			(const char *)"PEF::rmps_status()", Tango::ERR);	
	}
}

//+-------------------------------------------------------------
/**
 *	method:	TopupSB::Pause Fillpath
 *
 *	description:	method to execute "Pause Fillpath"
 *	
 *
 *
 */
//+------------------------------------------------------------------
void TopupSB::fillpath_pauseOn()
{
	Tango::DeviceAttribute Din("Pause",true);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Set Pause " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}
void TopupSB::fillpath_pauseOff()
{
	Tango::DeviceAttribute Din("Pause",false);
	try {	
		FillpathD->write_attribute(Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Set Pause " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::FillpathD(): " <<  out_stream.str() << endl;
		set_state(Tango::FAULT);	
		set_status(out_stream.str());
		abort_sleep(1);		
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::fillpath_stop
 *
 *	description:	method to stop fillpath
 *	
 *
 *
 */

//+------------------------------------------------------------------
void TopupSB::fillpath_stop()
{
	try {	
		FillpathD->command_inout("Stop");	
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Stop fillpath " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::fillpath_stop(): " <<  out_stream.str() << endl;
	}
}

//+------------------------------------------------------------------
/**
 *   method:   TopupSB::fillpath_stop
 *
 *   description:   method to stop fillpath
 *   
 *
 *
 */

//+------------------------------------------------------------------
void TopupSB::TopupOff()
{
     try {     
          TopupD->command_inout("Off");  
     }
     catch(Tango::DevFailed e) {
          TangoSys_MemStream out_stream;
          out_stream << "Stop Topup " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
          WARN_STREAM << "TopupSB::fillpath_stop(): " <<  out_stream.str() << endl;
     }
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::fillpath_start
 *
 *	description:	method to start fillpath
 *	
 *
 *
 */

//+------------------------------------------------------------------
void TopupSB::fillpath_start()
{
	try {	
		FillpathD->command_inout("Start");	
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Start fillpath " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::fillpath_start(): " <<  out_stream.str() << endl;
	}
}

//+------------------------------------------------------------------
/**
 *	method:	TopupSB::fillpath_rest
 *
 *	description:	method to reset fillpath
 *	
 *
 *
 */

//+------------------------------------------------------------------
void TopupSB::fillpath_reset()
{
	try {	
		FillpathD->command_inout("Reset");	
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "Reset fillpath " <<  FillpathD << " failed, err=" << e.errors[0].desc << ends;
		WARN_STREAM << "TopupSB::fillpath_reset(): " <<  out_stream.str() << endl;
	}
}


//+------------------------------------------------------------------
/**
 *	method:	TOPUPSB::abort_sleep
 *
 *	Resume from sleep if abort_flag set (sec.)
 */
//+------------------------------------------------------------------
void TopupSB::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.01); i++) {
		if (abortflag)
			break;
		omni_thread::sleep(0,10000000);
	}
}


	/*----- PROTECTED REGION ID(TopupSB::namespace_ending) ENABLED START -----*/

	//	Additional Methods

	/*----- PROTECTED REGION END -----*/	//	TopupSB::namespace_ending
} //	namespace
